/* This file is automatically generated from cssvalues.in by
#makevalues, do not edit */
/* Copyright 1998 W. Bastian */

#ifndef CSSVALUES_H
#define CSSVALUES_H

DOM::DOMString getValueName(unsigned short id);

#define CSS_VAL_INVALID 0
#define CSS_VAL_MIN 1
#define CSS_VAL_INHERIT 1
#define CSS_VAL_NONE 2
#define CSS_VAL_HIDDEN 3
#define CSS_VAL_DOTTED 4
#define CSS_VAL_DASHED 5
#define CSS_VAL_DOUBLE 6
#define CSS_VAL_SOLID 7
#define CSS_VAL_OUTSET 8
#define CSS_VAL_INSET 9
#define CSS_VAL_GROOVE 10
#define CSS_VAL_RIDGE 11
#define CSS_VAL_CAPTION 12
#define CSS_VAL_ICON 13
#define CSS_VAL_MENU 14
#define CSS_VAL_MESSAGE_BOX 15
#define CSS_VAL_SMALL_CAPTION 16
#define CSS_VAL_STATUS_BAR 17
#define CSS_VAL_ITALIC 18
#define CSS_VAL_OBLIQUE 19
#define CSS_VAL_SMALL_CAPS 20
#define CSS_VAL_NORMAL 21
#define CSS_VAL_BOLD 22
#define CSS_VAL_BOLDER 23
#define CSS_VAL_LIGHTER 24
#define CSS_VAL_100 25
#define CSS_VAL_200 26
#define CSS_VAL_300 27
#define CSS_VAL_400 28
#define CSS_VAL_500 29
#define CSS_VAL_600 30
#define CSS_VAL_700 31
#define CSS_VAL_800 32
#define CSS_VAL_900 33
#define CSS_VAL_XX_SMALL 34
#define CSS_VAL_X_SMALL 35
#define CSS_VAL_SMALL 36
#define CSS_VAL_MEDIUM 37
#define CSS_VAL_LARGE 38
#define CSS_VAL_X_LARGE 39
#define CSS_VAL_XX_LARGE 40
#define CSS_VAL__KHTML_XXX_LARGE 41
#define CSS_VAL_SMALLER 42
#define CSS_VAL_LARGER 43
#define CSS_VAL_WIDER 44
#define CSS_VAL_NARROWER 45
#define CSS_VAL_ULTRA_CONDENSED 46
#define CSS_VAL_EXTRA_CONDENSED 47
#define CSS_VAL_CONDENSED 48
#define CSS_VAL_SEMI_CONDENSED 49
#define CSS_VAL_SEMI_EXPANDED 50
#define CSS_VAL_EXPANDED 51
#define CSS_VAL_EXTRA_EXPANDED 52
#define CSS_VAL_ULTRA_EXPANDED 53
#define CSS_VAL_SERIF 54
#define CSS_VAL_SANS_SERIF 55
#define CSS_VAL_CURSIVE 56
#define CSS_VAL_FANTASY 57
#define CSS_VAL_MONOSPACE 58
#define CSS_VAL__KHTML_BODY 59
#define CSS_VAL_TRANSPARENT 60
#define CSS_VAL_AQUA 61
#define CSS_VAL_BLACK 62
#define CSS_VAL_BLUE 63
#define CSS_VAL_FUCHSIA 64
#define CSS_VAL_GRAY 65
#define CSS_VAL_GREEN 66
#define CSS_VAL_LIME 67
#define CSS_VAL_MAROON 68
#define CSS_VAL_NAVY 69
#define CSS_VAL_OLIVE 70
#define CSS_VAL_ORANGE 71
#define CSS_VAL_PURPLE 72
#define CSS_VAL_RED 73
#define CSS_VAL_SILVER 74
#define CSS_VAL_TEAL 75
#define CSS_VAL_WHITE 76
#define CSS_VAL_YELLOW 77
#define CSS_VAL_ACTIVEBORDER 78
#define CSS_VAL_ACTIVECAPTION 79
#define CSS_VAL_APPWORKSPACE 80
#define CSS_VAL_BACKGROUND 81
#define CSS_VAL_BUTTONFACE 82
#define CSS_VAL_BUTTONHIGHLIGHT 83
#define CSS_VAL_BUTTONSHADOW 84
#define CSS_VAL_BUTTONTEXT 85
#define CSS_VAL_CAPTIONTEXT 86
#define CSS_VAL_GRAYTEXT 87
#define CSS_VAL_HIGHLIGHT 88
#define CSS_VAL_HIGHLIGHTTEXT 89
#define CSS_VAL_INACTIVEBORDER 90
#define CSS_VAL_INACTIVECAPTION 91
#define CSS_VAL_INACTIVECAPTIONTEXT 92
#define CSS_VAL_INFOBACKGROUND 93
#define CSS_VAL_INFOTEXT 94
#define CSS_VAL_MENUTEXT 95
#define CSS_VAL_SCROLLBAR 96
#define CSS_VAL_THREEDDARKSHADOW 97
#define CSS_VAL_THREEDFACE 98
#define CSS_VAL_THREEDHIGHLIGHT 99
#define CSS_VAL_THREEDLIGHTSHADOW 100
#define CSS_VAL_THREEDSHADOW 101
#define CSS_VAL_WINDOW 102
#define CSS_VAL_WINDOWFRAME 103
#define CSS_VAL_WINDOWTEXT 104
#define CSS_VAL_GREY 105
#define CSS_VAL__KHTML_TEXT 106
#define CSS_VAL_REPEAT 107
#define CSS_VAL_REPEAT_X 108
#define CSS_VAL_REPEAT_Y 109
#define CSS_VAL_NO_REPEAT 110
#define CSS_VAL_BASELINE 111
#define CSS_VAL_MIDDLE 112
#define CSS_VAL_SUB 113
#define CSS_VAL_SUPER 114
#define CSS_VAL_TEXT_TOP 115
#define CSS_VAL_TEXT_BOTTOM 116
#define CSS_VAL_TOP 117
#define CSS_VAL_BOTTOM 118
#define CSS_VAL__KHTML_BASELINE_MIDDLE 119
#define CSS_VAL__KHTML_AUTO 120
#define CSS_VAL_LEFT 121
#define CSS_VAL_RIGHT 122
#define CSS_VAL_CENTER 123
#define CSS_VAL_JUSTIFY 124
#define CSS_VAL__KHTML_LEFT 125
#define CSS_VAL__KHTML_RIGHT 126
#define CSS_VAL__KHTML_CENTER 127
#define CSS_VAL_OUTSIDE 128
#define CSS_VAL_INSIDE 129
#define CSS_VAL_DISC 130
#define CSS_VAL_CIRCLE 131
#define CSS_VAL_SQUARE 132
#define CSS_VAL_DECIMAL 133
#define CSS_VAL_DECIMAL_LEADING_ZERO 134
#define CSS_VAL_LOWER_ROMAN 135
#define CSS_VAL_UPPER_ROMAN 136
#define CSS_VAL_LOWER_GREEK 137
#define CSS_VAL_LOWER_ALPHA 138
#define CSS_VAL_LOWER_LATIN 139
#define CSS_VAL_UPPER_ALPHA 140
#define CSS_VAL_UPPER_LATIN 141
#define CSS_VAL_HEBREW 142
#define CSS_VAL_ARMENIAN 143
#define CSS_VAL_GEORGIAN 144
#define CSS_VAL_CJK_IDEOGRAPHIC 145
#define CSS_VAL_HIRAGANA 146
#define CSS_VAL_KATAKANA 147
#define CSS_VAL_HIRAGANA_IROHA 148
#define CSS_VAL_KATAKANA_IROHA 149
#define CSS_VAL_INLINE 150
#define CSS_VAL_BLOCK 151
#define CSS_VAL_LIST_ITEM 152
#define CSS_VAL_RUN_IN 153
#define CSS_VAL_COMPACT 154
#define CSS_VAL_INLINE_BLOCK 155
#define CSS_VAL_TABLE 156
#define CSS_VAL_INLINE_TABLE 157
#define CSS_VAL_TABLE_ROW_GROUP 158
#define CSS_VAL_TABLE_HEADER_GROUP 159
#define CSS_VAL_TABLE_FOOTER_GROUP 160
#define CSS_VAL_TABLE_ROW 161
#define CSS_VAL_TABLE_COLUMN_GROUP 162
#define CSS_VAL_TABLE_COLUMN 163
#define CSS_VAL_TABLE_CELL 164
#define CSS_VAL_TABLE_CAPTION 165
#define CSS_VAL__KHTML_BOX 166
#define CSS_VAL__KHTML_INLINE_BOX 167
#define CSS_VAL_AUTO 168
#define CSS_VAL_CROSSHAIR 169
#define CSS_VAL_DEFAULT 170
#define CSS_VAL_POINTER 171
#define CSS_VAL_PROGRESS 172
#define CSS_VAL_MOVE 173
#define CSS_VAL_E_RESIZE 174
#define CSS_VAL_NE_RESIZE 175
#define CSS_VAL_NW_RESIZE 176
#define CSS_VAL_N_RESIZE 177
#define CSS_VAL_SE_RESIZE 178
#define CSS_VAL_SW_RESIZE 179
#define CSS_VAL_S_RESIZE 180
#define CSS_VAL_W_RESIZE 181
#define CSS_VAL_TEXT 182
#define CSS_VAL_WAIT 183
#define CSS_VAL_HELP 184
#define CSS_VAL_LTR 185
#define CSS_VAL_RTL 186
#define CSS_VAL_CAPITALIZE 187
#define CSS_VAL_UPPERCASE 188
#define CSS_VAL_LOWERCASE 189
#define CSS_VAL_VISIBLE 190
#define CSS_VAL_COLLAPSE 191
#define CSS_VAL_ABOVE 192
#define CSS_VAL_ABSOLUTE 193
#define CSS_VAL_ALWAYS 194
#define CSS_VAL_AVOID 195
#define CSS_VAL_BELOW 196
#define CSS_VAL_BIDI_OVERRIDE 197
#define CSS_VAL_BLINK 198
#define CSS_VAL_BOTH 199
#define CSS_VAL_CLOSE_QUOTE 200
#define CSS_VAL_CROP 201
#define CSS_VAL_CROSS 202
#define CSS_VAL_EMBED 203
#define CSS_VAL_FIXED 204
#define CSS_VAL_HAND 205
#define CSS_VAL_HIDE 206
#define CSS_VAL_HIGHER 207
#define CSS_VAL_INVERT 208
#define CSS_VAL_LANDSCAPE 209
#define CSS_VAL_LEVEL 210
#define CSS_VAL_LINE_THROUGH 211
#define CSS_VAL_LOUD 212
#define CSS_VAL_LOWER 213
#define CSS_VAL_MIX 214
#define CSS_VAL_NO_CLOSE_QUOTE 215
#define CSS_VAL_NO_OPEN_QUOTE 216
#define CSS_VAL_NOWRAP 217
#define CSS_VAL_OPEN_QUOTE 218
#define CSS_VAL_OVERLINE 219
#define CSS_VAL_PORTRAIT 220
#define CSS_VAL_PRE 221
#define CSS_VAL_RELATIVE 222
#define CSS_VAL_SCROLL 223
#define CSS_VAL_SEPARATE 224
#define CSS_VAL_SHOW 225
#define CSS_VAL_STATIC 226
#define CSS_VAL_THICK 227
#define CSS_VAL_THIN 228
#define CSS_VAL_UNDERLINE 229
#define CSS_VAL__KHTML_NOWRAP 230
#define CSS_VAL__KHTML_NORMAL 231
#define CSS_VAL__KHTML_AROUND_FLOATS 232
#define CSS_VAL_STRETCH 233
#define CSS_VAL_START 234
#define CSS_VAL_END 235
#define CSS_VAL_REVERSE 236
#define CSS_VAL_HORIZONTAL 237
#define CSS_VAL_VERTICAL 238
#define CSS_VAL_INLINE_AXIS 239
#define CSS_VAL_BLOCK_AXIS 240
#define CSS_VAL_SINGLE 241
#define CSS_VAL_MULTIPLE 242

#define CSS_VAL_TOTAL 243
#endif

