/* This file is automatically generated from
htmlattrs.in by makeattrs, do not edit */
/* Copyright 1999 Lars Knoll */

#ifndef HTML_ATTRS_H
#define HTML_ATTRS_H

#include "dom/dom_string.h"
using namespace DOM;

#define ATTR_ABBR 1
#define ATTR_ACCEPT_CHARSET 2
#define ATTR_ACCEPT 3
#define ATTR_ACCESSKEY 4
#define ATTR_ACTION 5
#define ATTR_ALIGN 6
#define ATTR_ALINK 7
#define ATTR_ALT 8
#define ATTR_ARCHIVE 9
#define ATTR_AUTOCOMPLETE 10
#define ATTR_AXIS 11
#define ATTR_BACKGROUND 12
#define ATTR_BGCOLOR 13
#define ATTR_BGPROPERTIES 14
#define ATTR_BORDER 15
#define ATTR_BORDERCOLOR 16
#define ATTR_CELLPADDING 17
#define ATTR_CELLSPACING 18
#define ATTR_CHAR 19
#define ATTR_CHALLENGE 20
#define ATTR_CHAROFF 21
#define ATTR_CHARSET 22
#define ATTR_CHECKED 23
#define ATTR_CITE 24
#define ATTR_CLASS 25
#define ATTR_CLASSID 26
#define ATTR_CLEAR 27
#define ATTR_CODE 28
#define ATTR_CODEBASE 29
#define ATTR_CODETYPE 30
#define ATTR_COLOR 31
#define ATTR_COLS 32
#define ATTR_COLSPAN 33
#define ATTR_COMPACT 34
#define ATTR_CONTENT 35
#define ATTR_COORDS 36
#define ATTR_DATA 37
#define ATTR_DATETIME 38
#define ATTR_DECLARE 39
#define ATTR_DEFER 40
#define ATTR_DIR 41
#define ATTR_DISABLED 42
#define ATTR_ENCTYPE 43
#define ATTR_FACE 44
#define ATTR_FOR 45
#define ATTR_FRAME 46
#define ATTR_FRAMEBORDER 47
#define ATTR_HEADERS 48
#define ATTR_HEIGHT 49
#define ATTR_HIDDEN 50
#define ATTR_HREF 51
#define ATTR_HREFLANG 52
#define ATTR_HSPACE 53
#define ATTR_HTML 54
#define ATTR_HTTP_EQUIV 55
#define ATTR_ID 56
#define ATTR_ISMAP 57
#define ATTR_LABEL 58
#define ATTR_LANG 59
#define ATTR_LANGUAGE 60
#define ATTR_LEFT 61
#define ATTR_LEFTMARGIN 62
#define ATTR_LINK 63
#define ATTR_LONGDESC 64
#define ATTR_MARGINHEIGHT 65
#define ATTR_MARGINWIDTH 66
#define ATTR_MAXLENGTH 67
#define ATTR_MEDIA 68
#define ATTR_METHOD 69
#define ATTR_MULTIPLE 70
#define ATTR_NAME 71
#define ATTR_NOHREF 72
#define ATTR_NORESIZE 73
#define ATTR_NOSAVE 74
#define ATTR_NOSHADE 75
#define ATTR_NOWRAP 76
#define ATTR_OBJECT 77
#define ATTR_ONABORT 78
#define ATTR_ONBLUR 79
#define ATTR_ONCHANGE 80
#define ATTR_ONCLICK 81
#define ATTR_ONCONTEXTMENU 82
#define ATTR_ONDBLCLICK 83
#define ATTR_ONERROR 84
#define ATTR_ONFOCUS 85
#define ATTR_ONKEYDOWN 86
#define ATTR_ONKEYPRESS 87
#define ATTR_ONKEYUP 88
#define ATTR_ONLOAD 89
#define ATTR_ONMOUSEDOWN 90
#define ATTR_ONMOUSEMOVE 91
#define ATTR_ONMOUSEOUT 92
#define ATTR_ONMOUSEOVER 93
#define ATTR_ONMOUSEUP 94
#define ATTR_ONRESET 95
#define ATTR_ONRESIZE 96
#define ATTR_ONSELECT 97
#define ATTR_ONSUBMIT 98
#define ATTR_ONUNLOAD 99
#define ATTR_OVERSRC 100
#define ATTR_PAGEX 101
#define ATTR_PAGEY 102
#define ATTR_PLAIN 103
#define ATTR_PLUGINPAGE 104
#define ATTR_PLUGINSPAGE 105
#define ATTR_PLUGINURL 106
#define ATTR_PROFILE 107
#define ATTR_PROMPT 108
#define ATTR_READONLY 109
#define ATTR_REL 110
#define ATTR_REV 111
#define ATTR_ROWS 112
#define ATTR_ROWSPAN 113
#define ATTR_RULES 114
#define ATTR_SCHEME 115
#define ATTR_SCOPE 116
#define ATTR_SCROLLING 117
#define ATTR_SELECTED 118
#define ATTR_SHAPE 119
#define ATTR_SIZE 120
#define ATTR_SPAN 121
#define ATTR_SRC 122
#define ATTR_STANDBY 123
#define ATTR_START 124
#define ATTR_STYLE 125
#define ATTR_SUMMARY 126
#define ATTR_TABINDEX 127
#define ATTR_TARGET 128
#define ATTR_TEXT 129
#define ATTR_TITLE 130
#define ATTR_TOP 131
#define ATTR_TOPMARGIN 132
#define ATTR_TYPE 133
#define ATTR_UNKNOWN 134
#define ATTR_USEMAP 135
#define ATTR_VALIGN 136
#define ATTR_VALUE 137
#define ATTR_VALUETYPE 138
#define ATTR_VERSION 139
#define ATTR_VISIBILITY 140
#define ATTR_VLINK 141
#define ATTR_VSPACE 142
#define ATTR_WIDTH 143
#define ATTR_WRAP 144
#define ATTR_Z_INDEX 145
#define ATTR_LAST_ATTR 145
DOMString getAttrName(unsigned short id);

#endif
